<?php

namespace Bydedi\Template\Filter;

/**
 * Class SSL
 */

class SSL
{
    /**
     * @var SSL
     */
    private static $instance;
    /**
     * @var array
     */
    private $attributes;

    /**
     * @var array
     */
    private $extensions;

    /**
     * @var array
     */
    private $forceReplace;


    private function __construct()
    {
        $this->extensions = array();
        $this->forceReplace = false;
        $this->attributes = array('src','data-original');
    }

    /**
     * Get instance of Boutique
     *
     * @return null|SSL
     */
    public static function getInstance()
    {
        if (!(self::$instance instanceof self)) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    /**
     * @return bool
     */
    public function isEnabled(){
        return (
            (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off')
            || (isset($_SERVER['SERVER_PORT']) && $_SERVER['SERVER_PORT'] == 443)
            || (isset($_SERVER['SERVER_PORT']) && stripos($_SERVER['SERVER_PROTOCOL'],'https') !== false)
            || (isset($_SERVER['HTTP_X_FORWARDED_PROTO']) && stripos($_SERVER['HTTP_X_FORWARDED_PROTO'],'https') !== false)
            || (isset($_SERVER['HTTP_X_FORWARDED_PORT]']) && $_SERVER['HTTP_X_FORWARDED_PORT]'] == 443)
        )
            ;
    }

    /**
     * @param array $attributes
     */
    public function setAttributes(Array $attributes)
    {
        $this->attributes = $attributes;
    }

    /**
     * @param SSLExtensionInterface $extension
     */
    public function addExtension(SSLExtensionInterface $extension)
    {
        $this->extensions[] = $extension;
    }

    /**
     * @param $string
     *
     * @return mixed
     */
    public function replaceHttpProtocol($string){
        if ($this->isEnabled() || $this->forceReplace) {
            $urlnossl = 'http:' . str_replace(array('https:', 'http:'), '', SITE_URL);
            $string = str_replace($urlnossl, str_replace('http:', 'https:', SITE_URL), $string);
            $string = preg_replace('#('.implode('|',$this->attributes).')=([\'"])http:#i', '$1=$2https:', $string);
            foreach ($this->extensions as $e){
                $string = $e->replace($string);
            }
        }
        return $string;
    }

    public function smartyOutputFilter($smarty)
    {
        if (property_exists($smarty, '_version') && substr($smarty->_version, 0, 1) == 2) {
            $smarty->register_outputfilter(array($this, 'smartyOutputFilterV2'));
        } else {
            $smarty->registerFilter('output', array($this, 'smartyOutputFilterV3'));
        }
    }

    /**
     * @param $output
     * @param $smarty
     *
     * @return mixed
     */
    public function smartyOutputFilterV3($output, \Smarty_Internal_Template $template)
    {
        return $this->replaceHttpProtocol($output);
    }

    public function smartyOutputFilterV2($output, $template)
    {
        return $this->replaceHttpProtocol($output);
    }

    /**
     *
     */
    public function redirect()
    {
        if (!$this->isEnabled() && strpos($_SERVER['HTTP_HOST'],'.dev') === false){
            header('Location: https://'.$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'],true,301);
            exit();
        }
    }

    public function setForceReplace($bool)
    {
        $this->forceReplace = (bool)$bool;
    }
}